# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"get_subs": "00_core.ipynb",
         "filter_workflow": "00_core.ipynb",
         "get_bfiles_tuple": "00_core.ipynb",
         "get_sub_gradfiles": "00_core.ipynb",
         "BIDS_metadata": "00_core.ipynb",
         "PipetographyBaseInputSpec": "00_core.ipynb",
         "MRCatInputSpec": "00_core.ipynb",
         "MRCatOutputSpec": "00_core.ipynb",
         "MRCat": "00_core.ipynb",
         "GradCatInputSpec": "00_core.ipynb",
         "GradCatOutputSpec": "00_core.ipynb",
         "GradCat": "00_core.ipynb",
         "aff2rigidInputSpec": "00_core.ipynb",
         "aff2rigidOutputSpec": "00_core.ipynb",
         "fslaff2rigid": "00_core.ipynb",
         "ConvertInputSpec": "00_core.ipynb",
         "ConvertOutputSpec": "00_core.ipynb",
         "Convert": "00_core.ipynb",
         "GradCheckInputSpec": "00_core.ipynb",
         "GradCheckOutputSpec": "00_core.ipynb",
         "GradCheck": "00_core.ipynb",
         "dwidenoiseInputSpec": "00_core.ipynb",
         "dwidenoiseOutputSpec": "00_core.ipynb",
         "dwidenoise": "00_core.ipynb",
         "dwipreprocInputSpec": "00_core.ipynb",
         "dwipreprocOutputSpec": "00_core.ipynb",
         "dwipreproc": "00_core.ipynb",
         "BiasCorrectInputSpec": "00_core.ipynb",
         "BiasCorrectOutputSpec": "00_core.ipynb",
         "BiasCorrect": "00_core.ipynb",
         "MRInfoInputSpec": "00_core.ipynb",
         "MRInfoOutputSpec": "00_core.ipynb",
         "MRInfo": "00_core.ipynb",
         "CheckFiniteInputSpec": "00_core.ipynb",
         "CheckFiniteOutputSpec": "00_core.ipynb",
         "CheckFinite": "00_core.ipynb",
         "CompareIfInputSpec": "00_core.ipynb",
         "CompareIfOutputSpec": "00_core.ipynb",
         "CompareIf": "00_core.ipynb",
         "CheckNIZInputSpec": "00_core.ipynb",
         "CheckNIZOutputSpec": "00_core.ipynb",
         "CheckNIZ": "00_core.ipynb",
         "RicianNoiseInputSpec": "00_core.ipynb",
         "RicianNoiseOutputSpec": "00_core.ipynb",
         "RicianNoise": "00_core.ipynb",
         "MRThresholdInputSpec": "00_core.ipynb",
         "MRThresholdOutputSpec": "00_core.ipynb",
         "MRThreshold": "00_core.ipynb",
         "DWINormalizeInputSpec": "00_core.ipynb",
         "DWINormalizeOutputSpec": "00_core.ipynb",
         "DWINormalize": "00_core.ipynb",
         "TransConvertInputSpec": "00_core.ipynb",
         "TransConvertOutputSpec": "00_core.ipynb",
         "TransConvert": "00_core.ipynb",
         "MRTransformInputSpec": "00_core.ipynb",
         "MRTransformOutputSpec": "00_core.ipynb",
         "MRTransform": "00_core.ipynb",
         "WMBinarizeInputSpec": "00_core.ipynb",
         "WMBinarizeOutputSpec": "00_core.ipynb",
         "WMBinarize": "00_core.ipynb",
         "MRRegridInputSpec": "00_core.ipynb",
         "MRRegridOutputSpec": "00_core.ipynb",
         "MRRegrid": "00_core.ipynb",
         "mask2seedtuple": "00_core.ipynb",
         "gmwmiInputSpec": "00_core.ipynb",
         "gmwmiOutputSpec": "00_core.ipynb",
         "gmwmi": "00_core.ipynb",
         "tckSIFT2InputSpec": "00_core.ipynb",
         "tckSIFT2OutputSpec": "00_core.ipynb",
         "tckSIFT2": "00_core.ipynb",
         "MakeConnectomeInputSpec": "00_core.ipynb",
         "MakeConnectomeOutputSpec": "00_core.ipynb",
         "MakeConnectome": "00_core.ipynb",
         "pipeline": "01_pipeline.ipynb",
         "PreProcNodes": "02_nodes.ipynb",
         "ACPCNodes": "02_nodes.ipynb",
         "PostProcNodes": "02_nodes.ipynb",
         "connectome": "03_connectomes.ipynb",
         "Make5ttFSLInputSpec": "00_core.ipynb",
         "Make5ttFSLOutputSpec": "00_core.ipynb",
         "Make5ttFSL": "00_core.ipynb"}

modules = ["core.py",
           "pipeline.py",
           "nodes.py",
           "connectomes.py"]

doc_url = "https://axiezai.github.io/pipetography/"

git_url = "https://github.com/axiezai/pipetography/tree/master/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: 01_pipeline.ipynb (unless otherwise specified).

__all__ = ['pipeline']

# Internal Cell
import os, sys
import bids

bids.config.set_option("extension_initial_dot", True)

from shutil import which
from nipype.pipeline import Workflow

import pipetography.core as ppt
import pipetography.nodes as nodes

# Cell
class pipeline:
    """
    Create a `Nipype` workflow, collects the layout of a BIDS directory, and branches the workflow into sub-graphs that can be ran in parallel per subject-session combination
    Inputs:
        - BIDS_dir (str): Path to BIDS dataset.
        - ext (str): Extension of your BIDS image files. Default is set to "nii.gz"
        - rpe_design (str): Reverse phase encoding design for your DWI acquisition. Also supports '-rpe_all', default is "-rpe_none"
        - regrid (bool): Whether  to resample DWI to  1mm MNI template, defaults to True.
        - mrtrix_nthreads (int): Number of threads for mrtrix3 algorithm. If zero, the number of available CPUs will be used. Default is 0.
        - skip_tuples (list[tuple]): A combination of [('subject #', 'session #')] tuples to skip, example: [('01', '03')] will skip sub-01/ses-03. Used for missinng data, the pipeline will automatically remove inconsistent sessions from BIDS Layout.
    """

    def __init__(
        self,
        BIDS_dir,
        ext="nii.gz",
        rpe_design="-rpe_none",
        regrid=True,
        mrtrix_nthreads=0,
        skip_tuples=[()],
    ):
        self.bids_dir = BIDS_dir
        self.rpe_design = rpe_design
        self.regrid = regrid
        self.mrtrix_nthreads = mrtrix_nthreads
        self.ext = ext
        self.excludes = skip_tuples
        self.MNI_template = os.path.expandvars(
            "$FSLDIR/data/standard/MNI152_T1_1mm.nii.gz"
        )
        # Create derivatives folder if it doesn't exist:
        if not os.path.exists(os.path.join(BIDS_dir, "derivatives")):
            print(
                "No derivatives folder found in BIDS dataset folder, creating it at {}".format(
                    os.path.join(BIDS_dir, "derivatives")
                )
            )
            os.makedirs(os.path.join(BIDS_dir, "derivatives"))
        elif os.path.exists(os.path.join(BIDS_dir, "derivatives")):
            print(
                "derivatives folder found at {}".format(
                    os.path.join(BIDS_dir, "derivatives")
                )
            )
        # Generate BIDS Layout, and create subject ID list:
        self.sub_list, self.ses_list, self.layout = ppt.get_subs(self.bids_dir)
        # string templates for images: the templates will depend on rpe_design:
        #  - rpe_none: anat_file, dwi_file, b_files
        #  - rpe_all/rpe_pair: anat_file, dwi_file, rdwi_file, b_files, rb_files
        if rpe_design == "-rpe_none":
            self.anat_file = os.path.join(
                "sub-{subject_id}",
                "ses-{session_id}",
                "anat",
                "sub-{subject_id}_ses-{session_id}_T1w." + ext,
            )
            self.dwi_file = os.path.join(
                "sub-{subject_id}",
                "ses-{session_id}",
                "dwi",
                "sub-{subject_id}_ses-{session_id}_dwi." + ext,
            )
            self.b_files = os.path.join(
                "sub-{subject_id}",
                "ses-{session_id}",
                "dwi",
                "sub-{subject_id}_ses-{session_id}_dwi.bv*",
            )
        elif rpe_design == "-rpe_all":
            # if all directions were acquired twice, with reverse phase encoding directions.
            self.anat_file = os.path.join(
                "sub-{subject_id}",
                "ses-{session_id}",
                "anat",
                "sub-{subject_id}_ses-{session_id}_T1w." + ext,
            )
            self.dwi_file = os.path.join(
                "sub-{subject_id}",
                "ses-{session_id}",
                "dwi",
                "sub-{subject_id}_ses-{session_id}_ap_dwi." + ext,
            )
            self.b_files = os.path.join(
                "sub-{subject_id}",
                "ses-{session_id}",
                "dwi",
                "sub-{subject_id}_ses-{session_id}_ap_dwi.bv*",
            )
            self.rdwi_file = os.path.join(
                "sub-{subject_id}",
                "ses-{session_id}",
                "dwi",
                "sub-{subject_id}_ses-{session_id}_pa_dwi." + ext,
            )
            self.rb_files = os.path.join(
                "sub-{subject_id}",
                "ses-{session_id}",
                "dwi",
                "sub-{subject_id}_ses-{session_id}_pa_dwi.bv*",
            )

    def create_nodes(self):
        if self.rpe_design == "-rpe_none":
            self.sub_template = {
                "anat": self.anat_file,
                "dwi": self.dwi_file,
                "b_files": self.b_files,
            }
        elif self.rpe_design == "-rpe_all":
            self.sub_template = {
                "anat": self.anat_file,
                "dwi": self.dwi_file,
                "bfiles": self.b_files,
                "rdwi": self.rdwi_file,
                "rbfiles": self.rb_files,
            }
        if self.regrid:
            resol = "1mm"
        else:
            resol = "orig"
        self.PreProcNodes = nodes.PreProcNodes(
            bids_dir=self.bids_dir,
            bids_path_template=self.sub_template,
            bids_ext=self.ext,
            rpe_design=self.rpe_design,
            img_resol=resol,
            mrtrix_nthreads=self.mrtrix_nthreads,
            sub_list=self.sub_list,
            ses_list=self.ses_list,
            exclude_list=self.excludes,
        )
        self.ACPCNodes = nodes.ACPCNodes(MNI_template=self.MNI_template)
        self.workflow = None

    def check_environment(self):
        """
        Check your computing environment for FSL environment variables `FSLOUTPUTTYPE` and `FSLDIR`
        Check if ANTs PATH is included in your environment
        Check if mrtrix3 is in your PATH
        Check your freesurfer envrionment variable
        Looks for matlab compiler run time in freesurfer directory: https://surfer.nmr.mgh.harvard.edu/fswiki/MatlabRuntime
        """
        assert "FSLOUTPUTTYPE" in os.environ
        assert "FSLDIR" in os.environ
        if "FSLOUTPUTTYPE" in os.environ:
            print("FSLOUTPUTTYPE is valid")
        else:
            sys.exit("FSLOUTPUTTYPE is not defined, make sure FSL is configured!")

        if "FSLDIR" in os.environ:
            print("FSLDIR is valid")
        else:
            sys.exit("FSLOUTPUTTYPE is not defined, make sure FSL is configured!")
        # ANTS:
        assert "ANTSPATH" in os.environ
        print("ANTS is valid")
        # mrtrix:
        assert which("dwifslpreproc") is not None
        print("mrtrix3 is valid")
        # freesurfer:
        assert "FREESURFER_HOME" in os.environ
        print("Freesurfer path found")
        # matlab compiler run time
        assert os.path.isdir(os.path.expandvars("$FREESURFER_HOME/MCRv80"))
        print("Matlab Compiler Runtime 2012 version in Freesurfer Path found. (MCRv80)")
        return None

    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
    # Connect the nodes we defined above into a workflow:
    def connect_nodes(
        self, rpe_design, recon=True, regrid=True, wf_name="pipetography"
    ):
        """
        Connect nodes one by one and create Nipype workflow
        Input: wfname (str): name your workflow, default is pipetography
        """
        # a workflow that will be filled in with connections:
        self.workflow = Workflow(
            name=wf_name,
            base_dir=os.path.join(self.bids_dir, "derivatives"),
        )
        # connections:
        self.workflow.connect(
            [
                (
                    self.PreProcNodes.subject_source,
                    self.PreProcNodes.select_files,
                    [("subject_id", "subject_id"), ("session_id", "session_id")],
                ),
                (
                    self.PreProcNodes.select_files,
                    self.ACPCNodes.reduceFOV,
                    [("anat", "in_file")],
                ),
                (
                    self.PreProcNodes.select_files,
                    self.ACPCNodes.ACPC_warp,
                    [("anat", "in_file")],
                ),
                (
                    self.ACPCNodes.reduceFOV,
                    self.ACPCNodes.xfminverse,
                    [("out_transform", "in_file")],
                ),
                (
                    self.ACPCNodes.xfminverse,
                    self.ACPCNodes.concatxfm,
                    [("out_file", "in_file")],
                ),
                (
                    self.ACPCNodes.reduceFOV,
                    self.ACPCNodes.flirt,
                    [("out_roi", "in_file")],
                ),
                (
                    self.ACPCNodes.flirt,
                    self.ACPCNodes.concatxfm,
                    [("out_matrix_file", "in_file2")],
                ),
                (
                    self.ACPCNodes.concatxfm,
                    self.ACPCNodes.alignxfm,
                    [("out_file", "in_file")],
                ),
                (
                    self.ACPCNodes.alignxfm,
                    self.ACPCNodes.ACPC_warp,
                    [("out_file", "premat")],
                ),
                (
                    self.ACPCNodes.ACPC_warp,
                    self.PreProcNodes.datasink,
                    [("out_file", "preprocessed.@t1_acpc_aligned")],
                ),
                ## Adding WM mask extraction to replace original recon all section
                ## WM Mask extraction section
                (
                    self.ACPCNodes.ACPC_warp,
                    self.ACPCNodes.t1_bet,
                    [("out_file", "in_file")],
                ),
                (
                    self.ACPCNodes.t1_bet,
                    self.ACPCNodes.gen_5tt,
                    [("out_file", "in_file")],
                ),
                (
                    self.ACPCNodes.gen_5tt,
                    self.ACPCNodes.convert2wm,
                    [("out_file", "in_file")],
                ),
                (
                    self.ACPCNodes.gen_5tt,
                    self.ACPCNodes.gmwmi,
                    [("out_file", "in_file")],
                ),
                (
                    self.ACPCNodes.gmwmi,
                    self.ACPCNodes.binarize_gmwmi,
                    [("out_file", "in_file")],
                ),
                (
                    self.ACPCNodes.gen_5tt,
                    self.PreProcNodes.datasink,
                    [("out_file", "preprocessed.@mr5tt")],
                ),
                (
                    self.ACPCNodes.binarize_gmwmi,
                    self.PreProcNodes.datasink,
                    [("out_file", "preprocessed.@gmwmi")],
                ),
                (
                    self.ACPCNodes.convert2wm,
                    self.PreProcNodes.datasink,
                    [("out_file", "preprocessed.@wm")],
                ),
                (
                    self.ACPCNodes.ACPC_warp,
                    self.ACPCNodes.epi_reg,
                    [("out_file", "t1_head")],
                ),
                (
                    self.ACPCNodes.t1_bet,
                    self.ACPCNodes.epi_reg,
                    [("out_file", "t1_brain")],
                ),
                (
                    self.PreProcNodes.sub_apply_mask,
                    self.ACPCNodes.epi_reg,
                    [("out_file", "epi")],
                ),
                (
                    self.ACPCNodes.epi_reg,
                    self.ACPCNodes.acpc_xfm,
                    [("epi2str_mat", "flirt_xfm")],
                ),
                (
                    self.PreProcNodes.sub_apply_mask,
                    self.ACPCNodes.acpc_xfm,
                    [("out_file", "flirt_in")],
                ),
                (
                    self.ACPCNodes.t1_bet,
                    self.ACPCNodes.acpc_xfm,
                    [("out_file", "flirt_ref")],
                ),
                (
                    self.ACPCNodes.acpc_xfm,
                    self.ACPCNodes.apply_xfm,
                    [("out_file", "linear_xfm")],
                ),
                (
                    self.PreProcNodes.norm_intensity,
                    self.ACPCNodes.apply_xfm,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.select_files,
                    self.PreProcNodes.get_metadata,
                    [("dwi", "path")],
                ),
                (
                    self.PreProcNodes.get_metadata,
                    self.PreProcNodes.fslpreproc,
                    [("ReadoutTime", "RO_time"), ("PE_DIR", "pe_dir")],
                ),
                (
                    self.PreProcNodes.createMask,
                    self.PreProcNodes.GradCheck,
                    [("out_file", "mask_file")],
                ),
                (
                    self.PreProcNodes.GradCheck,
                    self.PreProcNodes.NewGradMR,
                    [("out_bfile", "grad_file")],
                ),
                (
                    self.PreProcNodes.NewGradMR,
                    self.PreProcNodes.denoise,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.denoise,
                    self.PreProcNodes.degibbs,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.degibbs,
                    self.PreProcNodes.fslpreproc,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.GradCheck,
                    self.PreProcNodes.fslpreproc,
                    [("out_bfile", "grad_file")],
                ),
                (
                    self.PreProcNodes.fslpreproc,
                    self.PreProcNodes.GradUpdate,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.fslpreproc,
                    self.PreProcNodes.ModGrad,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.GradUpdate,
                    self.PreProcNodes.ModGrad,
                    [("out_bfile", "grad_file")],
                ),
                (
                    self.PreProcNodes.ModGrad,
                    self.PreProcNodes.UpdateMif,
                    [("out_bfile", "grad_file")],
                ),
                (
                    self.PreProcNodes.fslpreproc,
                    self.PreProcNodes.UpdateMif,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.UpdateMif,
                    self.PreProcNodes.NewMask,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.ModGrad,
                    self.PreProcNodes.NewMask,
                    [("out_bfile", "grad_file")],
                ),
                (
                    self.PreProcNodes.UpdateMif,
                    self.PreProcNodes.biascorrect,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.ModGrad,
                    self.PreProcNodes.biascorrect,
                    [("out_bfile", "grad_file")],
                ),
                (
                    self.PreProcNodes.NewMask,
                    self.PreProcNodes.biascorrect,
                    [("out_file", "in_mask")],
                ),
                (
                    self.PreProcNodes.biascorrect,
                    self.PreProcNodes.grad_info,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.ModGrad,
                    self.PreProcNodes.grad_info,
                    [("out_bfile", "grad_file")],
                ),
                (
                    self.PreProcNodes.denoise,
                    self.PreProcNodes.low_noise_map,
                    [("noise", "isfinite")],
                ),
                (
                    self.PreProcNodes.denoise,
                    self.PreProcNodes.low_noise_map,
                    [("noise", "cond_if")],
                ),
                (
                    self.PreProcNodes.low_noise_map,
                    self.PreProcNodes.rician_noise,
                    [("out_file", "lownoisemap")],
                ),
                (
                    self.PreProcNodes.biascorrect,
                    self.PreProcNodes.rician_noise,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.rician_noise,
                    self.PreProcNodes.check_rician,
                    [("out_file", "isfinite")],
                ),
                (
                    self.PreProcNodes.rician_noise,
                    self.PreProcNodes.check_rician,
                    [("out_file", "cond_if")],
                ),
                (
                    self.PreProcNodes.check_rician,
                    self.PreProcNodes.convert_rician,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.grad_info,
                    self.PreProcNodes.convert_rician,
                    [("out_bfile", "grad_file")],
                ),
                (
                    self.PreProcNodes.convert_rician,
                    self.PreProcNodes.dwi_mask,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.dwi_mask,
                    self.PreProcNodes.fit_tensor,
                    [("out_file", "in_mask")],
                ),
                (
                    self.PreProcNodes.convert_rician,
                    self.PreProcNodes.fit_tensor,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.fit_tensor,
                    self.PreProcNodes.tensor_FA,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.tensor_FA,
                    self.PreProcNodes.wm_mask,
                    [("out_fa", "in_file")],
                ),
                (
                    self.PreProcNodes.wm_mask,
                    self.PreProcNodes.norm_intensity,
                    [("out_file", "mask_file")],
                ),
                (
                    self.PreProcNodes.convert_rician,
                    self.PreProcNodes.norm_intensity,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.norm_intensity,
                    self.PreProcNodes.sub_b0extract,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.sub_b0extract,
                    self.PreProcNodes.sub_b0mean,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.norm_intensity,
                    self.PreProcNodes.sub_b0mask,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.sub_b0mask,
                    self.PreProcNodes.sub_convert_mask,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.sub_b0mean,
                    self.PreProcNodes.sub_convert_dwi,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.sub_convert_dwi,
                    self.PreProcNodes.sub_apply_mask,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.sub_convert_mask,
                    self.PreProcNodes.sub_apply_mask,
                    [("out_file", "mask_file")],
                ),
                (
                    self.PreProcNodes.mni_b0extract,
                    self.PreProcNodes.mni_b0mean,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.mni_b0mask,
                    self.PreProcNodes.mni_convert_mask,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.mni_b0mean,
                    self.PreProcNodes.mni_convert_dwi,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.mni_convert_mask,
                    self.PreProcNodes.mni_apply_mask,
                    [("out_file", "mask_file")],
                ),
                (
                    self.PreProcNodes.mni_convert_dwi,
                    self.PreProcNodes.mni_apply_mask,
                    [("out_file", "in_file")],
                ),
                (
                    self.PreProcNodes.mni_apply_mask,
                    self.PreProcNodes.datasink,
                    [("out_file", "preprocessed.@dwi_brain")],
                ),
                (
                    self.PreProcNodes.mni_convert_mask,
                    self.PreProcNodes.datasink,
                    [("out_file", "preprocessed.@dwi_b0_brainmask")],
                ),
                (
                    self.PreProcNodes.mni_convert_dwi,
                    self.PreProcNodes.datasink,
                    [("out_file", "preprocessed.@dwi_b0_meanvolume")],
                ),
                (
                    self.PreProcNodes.mni_dwi,
                    self.PreProcNodes.datasink,
                    [("out_file", "preprocessed.@dwi")],
                ),
                (
                    self.PreProcNodes.mni_dwi,
                    self.PreProcNodes.datasink,
                    [("out_bfile", "preprocessed.@b")],
                ),
                (
                    self.PreProcNodes.mni_dwi,
                    self.PreProcNodes.datasink,
                    [("out_fslbvec", "preprocessed.@fsl_bvec")],
                ),
                (
                    self.PreProcNodes.mni_dwi,
                    self.PreProcNodes.datasink,
                    [("out_fslbval", "preprocessed.@fsl_bval")],
                ),
                (
                    self.PreProcNodes.mni_dwi,
                    self.PreProcNodes.datasink,
                    [("out_json", "preprocessed.@json")],
                ),
            ]
        )

        if rpe_design == "-rpe_none":
            self.workflow.connect(
                [
                    (
                        self.PreProcNodes.select_files,
                        self.PreProcNodes.mrconvert,
                        [("dwi", "in_file")],
                    ),
                    (
                        self.PreProcNodes.select_files,
                        self.PreProcNodes.sub_grad_files,
                        [("dwi", "sub_dwi")],
                    ),
                    (
                        self.PreProcNodes.sub_grad_files,
                        self.PreProcNodes.mrconvert,
                        [("fslgrad", "grad_fsl")],
                    ),
                    (
                        self.PreProcNodes.mrconvert,
                        self.PreProcNodes.createMask,
                        [("out_file", "in_file")],
                    ),
                    (
                        self.PreProcNodes.mrconvert,
                        self.PreProcNodes.GradCheck,
                        [("out_file", "in_file")],
                    ),
                    (
                        self.PreProcNodes.mrconvert,
                        self.PreProcNodes.GradCheck,
                        [("out_bfile", "grad_file")],
                    ),
                    (
                        self.PreProcNodes.mrconvert,
                        self.PreProcNodes.NewGradMR,
                        [("out_file", "in_file")],
                    ),
                    (
                        self.PreProcNodes.fslpreproc,
                        self.PreProcNodes.GradUpdate,
                        [("out_bfile", "grad_file")],
                    ),
                ]
            )
        elif rpe_design == "-rpe_all":
            self.workflow.connect(
                [
                    (
                        self.PreProcNodes.select_files,
                        self.PreProcNodes.sub_grad_files1,
                        [("dwi", "sub_dwi")],
                    ),
                    (
                        self.PreProcNodes.select_files,
                        self.PreProcNodes.sub_grad_files2,
                        [("rdwi", "sub_dwi")],
                    ),
                    (
                        self.PreProcNodes.select_files,
                        self.PreProcNodes.mrconvert1,
                        [("dwi", "in_file")],
                    ),
                    (
                        self.PreProcNodes.select_files,
                        self.PreProcNodes.mrconvert2,
                        [("rdwi", "in_file")],
                    ),
                    (
                        self.PreProcNodes.sub_grad_files1,
                        self.PreProcNodes.mrconvert1,
                        [("fslgrad", "grad_fsl")],
                    ),
                    (
                        self.PreProcNodes.sub_grad_files2,
                        self.PreProcNodes.mrconvert2,
                        [("fslgrad", "grad_fsl")],
                    ),
                    (
                        self.PreProcNodes.mrconvert1,
                        self.PreProcNodes.mrconcat,
                        [("out_file", "image1")],
                    ),
                    (
                        self.PreProcNodes.mrconvert2,
                        self.PreProcNodes.mrconcat,
                        [("out_file", "image2")],
                    ),
                    (
                        self.PreProcNodes.mrconvert1,
                        self.PreProcNodes.gradcat,
                        [("out_bfile", "grad1")],
                    ),
                    (
                        self.PreProcNodes.mrconvert2,
                        self.PreProcNodes.gradcat,
                        [("out_bfile", "grad2")],
                    ),
                    (
                        self.PreProcNodes.mrconcat,
                        self.PreProcNodes.createMask,
                        [("out_file", "in_file")],
                    ),
                    (
                        self.PreProcNodes.mrconcat,
                        self.PreProcNodes.GradCheck,
                        [("out_file", "in_file")],
                    ),
                    (
                        self.PreProcNodes.gradcat,
                        self.PreProcNodes.GradCheck,
                        [("out_file", "grad_file")],
                    ),
                    (
                        self.PreProcNodes.mrconcat,
                        self.PreProcNodes.NewGradMR,
                        [("out_file", "in_file")],
                    ),
                    (
                        self.PreProcNodes.mrconvert1,
                        self.PreProcNodes.GradUpdate,
                        [("out_bfile", "grad_file")],
                    ),
                ]
            )
        if regrid == True:
            self.workflow.connect(
                [
                    (
                        self.ACPCNodes.apply_xfm,
                        self.ACPCNodes.regrid,
                        [("out_file", "in_file")],
                    ),
                    (
                        self.ACPCNodes.regrid,
                        self.PreProcNodes.datasink,
                        [("out_file", "preprocessed.@dwi_acpc_aligned_1mm")],
                    ),
                    (
                        self.ACPCNodes.regrid,
                        self.PreProcNodes.mni_b0extract,
                        [("out_file", "in_file")],
                    ),
                    (
                        self.ACPCNodes.regrid,
                        self.PreProcNodes.mni_b0mask,
                        [("out_file", "in_file")],
                    ),
                    (
                        self.ACPCNodes.regrid,
                        self.PreProcNodes.mni_dwi,
                        [("out_file", "in_file")],
                    ),
                ]
            )
        elif regrid == False:
            self.workflow.connect(
                [
                    (
                        self.ACPCNodes.apply_xfm,
                        self.PreProcNodes.mni_b0extract,
                        [("out_file", "in_file")],
                    ),
                    (
                        self.ACPCNodes.apply_xfm,
                        self.PreProcNodes.mni_b0mask,
                        [("out_file", "in_file")],
                    ),
                    (
                        self.ACPCNodes.apply_xfm,
                        self.PreProcNodes.mni_dwi,
                        [("out_file", "in_file")],
                    ),
                ]
            )
        self.workflow.config["execution"] = {
            "use_relative_paths": "True",
            "hash_method": "content",
            "stop_on_first_crash": "True",
        }

    def draw_pipeline(self, graph_type="orig"):
        """
        Writes a `.png` image visualizing the workflow.
        Args:
            graph_type (str): Select from `orig`, `flat`, `hierarchical`, `colored`, or `exec`. See Nipype documentation for details. Defaults to orig.
        """
        self.workflow.write_graph(
            graph2use=graph_type,
            dotfilename=os.path.join(
                self.bids_dir, "code", "pipetography", "graph", "preprocessing.dot"
            ),
        )

    def run_pipeline(self, parallel=None):
        if type(parallel) == int:
            print("Running workflow with {} parallel processes".format(parallel))
            self.workflow.run("MultiProc", plugin_args={"n_procs": parallel})
        elif parallel is None:
            print("Parallel processing is not enabled, running workflow serially.")
            self.workflow.run()


# AUTOGENERATED! DO NOT EDIT! File to edit: 03_connectomes.ipynb (unless otherwise specified).

__all__ = ['connectome']

# Internal Cell
import os
import bids
bids.config.set_option('extension_initial_dot', True)

from nipype.pipeline import Node, Workflow

import pipetography.nodes as nodes
import pipetography.core as ppt

# Cell

class connectome:
    """
    Create a pipeline that produces connectomes based on input atlases and streamlines, the pipeline will create sub-graphs based on inputs BIDS directory subject & session combinations.

    Inputs:
         - BIDS_dir (str): base BIDS directory path
         - atlas_list (List of strings): names of atlases: aal, brainnectome, desikan-killiany, default is set to brainnectome for now.
    """

    def __init__(self, BIDS_dir, atlas_list, skip_tuples=[()]):
        """
        Initialize workflow nodes
        """
        self.bids_dir = BIDS_dir
        self.atlas_list = atlas_list
        self.sub_list, self.ses_list, self.layout = ppt.get_subs(BIDS_dir)
        self.skip_combos = skip_tuples
        self.subject_template = {
            'tck': os.path.join(self.bids_dir, 'derivatives', 'streamlines','sub-{subject_id}', 'ses-{session_id}', 'sub-{subject_id}_ses-{session_id}_gmwmi2wm.tck'),
            'brain': os.path.join(self.bids_dir, 'derivatives', 'pipetography', 'sub-{subject_id}', 'ses-{session_id}', 'preprocessed', 'dwi_space-acpc_res-1mm_seg-brain.nii.gz'),
            'dwi_mif': os.path.join(self.bids_dir, 'derivatives', 'pipetography', 'sub-{subject_id}', 'ses-{session_id}', 'preprocessed', 'dwi_space-acpc_res-1mm.mif'),
            'T1A': os.path.join(self.bids_dir, 'derivatives', 'pipetography', 'sub-{subject_id}', 'ses-{session_id}', 'preprocessed', 'T1w_space-acpc.nii.gz'),
            'mask': os.path.join(self.bids_dir, 'derivatives', 'pipetography', 'sub-{subject_id}', 'ses-{session_id}', 'preprocessed', 'dwi_space-acpc_res-1mm_seg-brain_mask.nii.gz'),
            'mrtrix5tt': os.path.join(self.bids_dir, 'derivatives', 'pipetography', 'sub-{subject_id}', 'ses-{session_id}', 'preprocessed', 'T1w_space-acpc_seg-5tt.mif')
        }


    def create_nodes(self):
        """
        Create postprocessing nodes, and make output path substitutions so outputs are BIDS compliant.
        """
        self.PostProcNodes = nodes.PostProcNodes(
            BIDS_dir=self.bids_dir,
            subj_template = self.subject_template,
            sub_list = self.sub_list,
            ses_list = self.ses_list,
            skip_tuples = self.skip_combos)
        self.PostProcNodes.linear_reg.iterables = [('moving_image', self.atlas_list)]
        self.workflow = None


    def connect_nodes(self, wf_name="connectomes"):
        """
        Connect postprocessing nodes into workflow
        """
        self.workflow = Workflow(name=wf_name, base_dir=os.path.join(self.bids_dir, 'derivatives'))
        self.workflow.connect(
            [
                (self.PostProcNodes.subject_source, self.PostProcNodes.select_files, [('subject_id', 'subject_id'),
                                                                                      ('session_id', 'session_id')]),
                (self.PostProcNodes.select_files, self.PostProcNodes.linear_reg, [('brain', 'fixed_image')]),
                (self.PostProcNodes.linear_reg, self.PostProcNodes.nonlinear_reg, [('warped_image', 'moving_image')]),
                (self.PostProcNodes.select_files, self.PostProcNodes.nonlinear_reg, [('brain', 'fixed_image')]),
                (self.PostProcNodes.nonlinear_reg, self.PostProcNodes.round_atlas, [('warped_image', 'in_file')]),
                (self.PostProcNodes.round_atlas, self.PostProcNodes.connectome, [('out_file', 'in_parc')]),
                (self.PostProcNodes.round_atlas, self.PostProcNodes.distance, [('out_file', 'in_parc')]),
                (self.PostProcNodes.select_files, self.PostProcNodes.response, [('dwi_mif', 'in_file')]),
                (self.PostProcNodes.select_files, self.PostProcNodes.fod, [('dwi_mif', 'in_file')]),
                (self.PostProcNodes.select_files, self.PostProcNodes.fod, [('mask', 'mask_file')]),
                (self.PostProcNodes.response, self.PostProcNodes.fod, [('wm_file', 'wm_txt')]),
                (self.PostProcNodes.response, self.PostProcNodes.fod, [('gm_file', 'gm_txt')]),
                (self.PostProcNodes.response, self.PostProcNodes.fod, [('csf_file', 'csf_txt')]),
                (self.PostProcNodes.select_files, self.PostProcNodes.sift2, [('mrtrix5tt', 'act')]),
                (self.PostProcNodes.select_files, self.PostProcNodes.sift2, [('tck', 'in_file')]),
                (self.PostProcNodes.fod, self.PostProcNodes.sift2, [('wm_odf', 'in_fod')]),
                (self.PostProcNodes.sift2, self.PostProcNodes.connectome, [('out_file', 'in_weights')]),
                (self.PostProcNodes.select_files, self.PostProcNodes.connectome, [('tck', 'in_file')]),
                (self.PostProcNodes.select_files, self.PostProcNodes.distance, [('tck', 'in_file')]),
                (self.PostProcNodes.connectome, self.PostProcNodes.datasink, [('out_file', 'connectomes.@connectome')]),
                (self.PostProcNodes.distance, self.PostProcNodes.datasink, [('out_file', 'connectomes.@distance')])
            ])
        self.workflow.config['execution'] = {
                                            'use_relative_paths':'True',
                                            'hash_method': 'content',
                                            'stop_on_first_crash': 'True',
                                            }

    def draw_pipeline(self, graph_type='orig'):
        """
        Visualize workflow
        """
        self.workflow.write_graph(
            graph2use=graph_type,
            dotfilename = os.path.join(
                self.bids_dir, 'derivatives', 'pipetography', 'graph', 'postprocessing.dot'
            ),
        )

    def run_pipeline(self, parallel=None):
        """
        Run nipype workflow
        """
        if type(parallel) == int:
            print("Running workflow with {} parallel processes".format(parallel))
            self.workflow.run('MultiProc', plugin_args = {'n_procs': parallel})
        elif parallel is None:
            print("Parallel processing disabled, running workflow serially")
            self.workflow.run()


# AUTOGENERATED! DO NOT EDIT! File to edit: 02_nodes.ipynb (unless otherwise specified).

__all__ = ['PreProcNodes', 'ACPCNodes', 'PostProcNodes']

# Internal Cell
import os
import bids

bids.config.set_option("extension_initial_dot", True)

from pathlib import Path
from itertools import product

import pipetography.core as ppt

from nipype import IdentityInterface, Function
from nipype.interfaces.io import SelectFiles, DataSink
from nipype.pipeline import Node, MapNode, Workflow
from nipype.interfaces.mrtrix3.utils import (
    BrainMask,
    TensorMetrics,
    DWIExtract,
    MRMath,
)
from nipype.interfaces.mrtrix3.preprocess import MRDeGibbs, DWIBiasCorrect, ResponseSD
from nipype.interfaces.mrtrix3.reconst import (
    FitTensor,
    EstimateFOD,
    ConstrainedSphericalDeconvolution,
)
from nipype.interfaces import ants
from nipype.interfaces import fsl

# Cell
class PreProcNodes:
    """
    Initiate DWI preprocessing pipeline nodes.

    Inputs:
        - bids_dir (str) -
        - bids_path_template (dict) - template for file naming conventions
        - bids_ext (str) -
        - rpe_design (str)
        - mrtrix_nthreads (int)
        - img_resol (str)
        - sub_list (List)
        - ses_list (List)
        - exclude_list (tuple)
    """

    def __init__(
        self,
        bids_dir,
        bids_path_template,
        bids_ext,
        rpe_design,
        mrtrix_nthreads,
        img_resol,
        sub_list,
        ses_list,
        exclude_list=[()],
    ):
        # filtter & create sub-graphs for subjects and sessions combos
        sub_iter, ses_iter = ppt.filter_workflow(bids_dir, sub_list, ses_list, exclude_list)
        # Create BIDS output folder list for datasink
        BIDSFolders = [
            (
                "preprocessed/ses-%ssub-%s" % (session, subject),
                "sub-%s/ses-%s/preprocessed" % (subject, session),
            )
            for session in ses_list
            for subject in sub_list
        ]

        self.subject_source = Node(
            IdentityInterface(fields=["subject_id", "session_id"]),
            iterables=[("subject_id", sub_iter), ("session_id", ses_iter)],
            synchronize=True,
            name="sub_source",
        )
        self.subject_source.inputs.ext = bids_ext

        # reverse phase encoding design selection
        if rpe_design == "-rpe_none":
            self.sub_grad_files = Node(
                Function(
                    input_names=["sub_dwi", "ext"],
                    output_names=["fslgrad"],
                    function=ppt.get_sub_gradfiles,
                    ext=bids_ext,
                ),
                name="sub_grad_files",
            )
            self.sub_grad_files.inputs.ext = bids_ext
            self.mrconvert = Node(
                ppt.Convert(
                    out_file="raw_dwi.mif",
                    export_grad="raw_dwi.b",
                    nthreads=mrtrix_nthreads,
                ),
                name="mrtrix_image",
            )
        elif rpe_design == "-rpe_all":
            self.sub_grad_files1 = Node(
                Function(
                    input_names=["sub_dwi", "ext"],
                    output_names=["fslgrad"],
                    function=ppt.get_sub_gradfiles,
                    ext=bids_ext,
                ),
                name="sub_grad_files1",
            )
            self.sub_grad_files2 = Node(
                Function(
                    input_names=["sub_dwi", "ext"],
                    output_names=["fslgrad"],
                    function=ppt.get_sub_gradfiles,
                    ext=bids_ext,
                ),
                name="sub_grad_files2",
            )
            self.sub_grad_files1.inputs.ext = bids_ext
            self.sub_grad_files2.inputs.ext = bids_ext
            self.mrconvert1 = Node(
                ppt.Convert(
                    out_file="raw_dwi1.mif",
                    export_grad="raw_dwi1.b",
                    nthreads=mrtrix_nthreads,
                ),
                name="mrtrix_image1",
            )
            self.mrconvert2 = Node(
                ppt.Convert(
                    out_file="raw_dwi2.mif",
                    export_grad="raw_dwi2.b",
                    nthreads=mrtrix_nthreads,
                ),
                name="mrtrix_image2",
            )
            # concatenate the two images and their gradient files.
            self.mrconcat = Node(ppt.MRCat(out_file="raw_dwi.mif"), name="concat_dwi",)
            self.gradcat = Node(ppt.GradCat(out_file="raw_dwi.b"), name="concat_grad",)

        self.select_files = Node(
            SelectFiles(bids_path_template, base_directory=bids_dir),
            name="select_files",
        )

        self.get_metadata = Node(
            Function(
                input_names=["path", "bids_dir"],
                output_names=["ReadoutTime", "PE_DIR"],
                function=ppt.BIDS_metadata,
            ),
            name="get_metadata",
        )
        self.get_metadata.inputs.bids_dir = bids_dir

        self.createMask = Node(
            BrainMask(out_file="b0_brain_mask.mif", nthreads=mrtrix_nthreads),
            name="raw_dwi2mask",
        )

        self.GradCheck = Node(
            ppt.GradCheck(export_grad="corrected.b", nthreads=mrtrix_nthreads),
            name="dwigradcheck",
        )

        self.NewGradMR = Node(
            ppt.Convert(out_file="corrected_dwi.mif", nthreads=mrtrix_nthreads),
            name="mrconvert",
        )

        self.denoise = Node(
            ppt.dwidenoise(
                out_file="denoised.mif", noise="noise_map.mif", nthreads=mrtrix_nthreads
            ),
            name="denoise",
        )

        self.degibbs = Node(
            MRDeGibbs(out_file="unring.mif", nthreads=mrtrix_nthreads),
            name="ringing_removal",
        )

        self.fslpreproc = Node(
            ppt.dwipreproc(
                out_file="preproc.mif",
                rpe_options=rpe_design,
                eddy_options='"--slm=linear --repol "',
                nthreads=mrtrix_nthreads,
                export_grad="eddy_dwi.b",
            ),
            name="dwifslpreproc",
        )

        self.GradUpdate = Node(
            ppt.GradCheck(export_grad="tmp.b"), name="alter_gradient"
        )

        self.ModGrad = Node(
            ppt.MRInfo(export_grad="modified.b"), name="modify_gradient"
        )

        self.UpdateMif = Node(ppt.Convert(), name="update_image")

        self.NewMask = Node(BrainMask(), name="recreate_mask")

        self.biascorrect = Node(
            ppt.BiasCorrect(
                use_ants=True,
                out_file="dwi_bias.mif",
                bias="biasfield.mif",
                nthreads=mrtrix_nthreads,
            ),
            name="dwibiascorrect",
        )

        self.grad_info = Node(
            ppt.MRInfo(export_grad="rician_tmp.b", nthreads=mrtrix_nthreads),
            name="NewGradient",
        )

        self.low_noise_map = Node(
            ppt.CheckNIZ(out_file="lownoisemap.mif", nthreads=mrtrix_nthreads),
            name="LowNoiseMap",
        )

        self.rician_noise = Node(
            ppt.RicianNoise(
                power=2,
                denoise=2,
                out_file="rician_removed_dwi.mif",
                nthreads=mrtrix_nthreads,
            ),
            name="RicianNoise",
        )

        self.check_rician = Node(
            ppt.CheckNIZ(out_file="rician_tmp.mif", nthreads=mrtrix_nthreads),
            name="NoiseComparison",
        )

        self.convert_rician = Node(
            ppt.Convert(out_file="rician_corrected_dwi.mif", nthreads=mrtrix_nthreads),
            name="ConvertRician",
        )
        self.dwi_mask = Node(BrainMask(out_file="dwi_mask.mif"), name="dwi2mask",)
        self.fit_tensor = Node(FitTensor(out_file="dti.mif"), name="dwi2tensor",)
        self.tensor_FA = Node(TensorMetrics(out_fa="fa.mif"), name="tensor2metrics",)
        self.wm_mask = Node(
            ppt.MRThreshold(opt_abs=0.5, out_file="wm.mif", nthreads=mrtrix_nthreads),
            name="mrthreshold",
        )
        self.norm_intensity = Node(
            ppt.DWINormalize(
                opt_intensity=1000,
                out_file="dwi_norm_intensity.mif",
                nthreads=mrtrix_nthreads,
            ),
            name="dwinormalise",
        )
        self.sub_b0extract = Node(
            DWIExtract(bzero=True, out_file="b0_volume.mif", nthreads=mrtrix_nthreads),
            name="sub_b0extract",
        )
        self.sub_b0mean = Node(
            MRMath(
                operation="mean",
                axis=3,
                out_file="b0_dwi.mif",
                nthreads=mrtrix_nthreads,
            ),
            name="sub_mrmath_mean",
        )
        self.sub_b0mask = Node(
            BrainMask(out_file="dwi_norm_mask.mif", nthreads=mrtrix_nthreads),
            name="sub_dwi2mask",
        )
        self.sub_convert_dwi = Node(
            ppt.Convert(out_file="b0_dwi.nii.gz"), name="sub_dwi2nii",
        )
        self.sub_convert_mask = Node(
            ppt.Convert(out_file="dwi_norm_mask.nii.gz"), name="sub_mask2nii",
        )
        self.sub_apply_mask = Node(
            fsl.ApplyMask(out_file="b0_dwi_brain.nii.gz"), name="sub_ApplyMask",
        )
        self.mni_b0extract = Node(
            DWIExtract(
                bzero=True,
                out_file="dwi_space-acpc_res-{}_b0.mif".format(img_resol),
                nthreads=mrtrix_nthreads,
            ),
            name="mni_b0extract",
        )
        self.mni_b0mean = Node(
            MRMath(
                operation="mean",
                axis=3,
                out_file="dwi_space-acpc_res-{}_b0mean.mif".format(img_resol),
                nthreads=mrtrix_nthreads,
            ),
            name="mni_mrmath_mean",
        )
        self.mni_b0mask = Node(
            BrainMask(
                out_file="dwi_space-acpc_res-{}_mask.mif".format(img_resol),
                nthreads=mrtrix_nthreads,
            ),
            name="mni_dwi2mask",
        )
        self.mni_convert_dwi = Node(
            ppt.Convert(
                out_file="dwi_space-acpc_res-{}_b0mean.nii.gz".format(img_resol)
            ),
            name="mni_dwi2nii",
        )
        self.mni_convert_mask = Node(
            ppt.Convert(
                out_file="dwi_space-acpc_res-{}_seg-brain_mask.nii.gz".format(img_resol)
            ),
            name="mni_mask2nii",
        )
        self.mni_apply_mask = Node(
            fsl.ApplyMask(
                out_file="dwi_space-acpc_res-{}_seg-brain.nii.gz".format(img_resol)
            ),
            name="mni_ApplyMask",
        )
        self.mni_dwi = Node(
            ppt.Convert(
                out_file="dwi_space-acpc_res-{}.nii.gz".format(img_resol),
                export_grad="dwi_space-acpc_res-{}.b".format(img_resol),
                export_fslgrad=(
                    "dwi_space-acpc_res-{}.bvecs".format(img_resol),
                    "dwi_space-acpc_res-{}.bvals".format(img_resol),
                ),
                export_json=True,
                nthreads=mrtrix_nthreads,
                out_json="dwi_space-acpc_res-{}.json".format(img_resol),
            ),
            name="MNI_Outputs",
        )

        self.datasink = Node(
            DataSink(
                base_directory=os.path.join(bids_dir, "derivatives", "pipetography")
            ),
            name="datasink",
        )
        substitutions = [("_subject_id_", "sub-"), ("_session_id_", "ses-")]
        substitutions.extend(BIDSFolders)
        self.datasink.inputs.substitutions = substitutions
        print(
            "Data sink (output folder) is set to {}".format(
                os.path.join(bids_dir, "derivatives", "pipetography")
            )
        )

# Cell
class ACPCNodes:
    """
    T1 anatomy image related nodes. Mainly ACPC alignment of T1 and DWI and extraction of white matter mask.
    Inputs:
        - MNI_template: path to MNI template provided by FSL. By default uses the environment variable FSLDIR to locate the reference templates for ACPC alignment.
    """

    def __init__(self, MNI_template):
        self.reduceFOV = Node(
            fsl.utils.RobustFOV(
                out_transform="roi2full.mat", out_roi="robustfov.nii.gz"
            ),
            name="reduce_FOV",
        )
        self.xfminverse = Node(
            fsl.utils.ConvertXFM(out_file="full2roi.mat", invert_xfm=True),
            name="transform_inverse",
        )
        self.flirt = Node(
            fsl.preprocess.FLIRT(
                reference=MNI_template,
                interp="spline",
                out_matrix_file="roi2std.mat",
                out_file="acpc_mni.nii.gz",
            ),
            name="FLIRT",
        )
        self.concatxfm = Node(
            fsl.utils.ConvertXFM(concat_xfm=True, out_file="full2std.mat"),
            name="concat_transform",
        )

        self.alignxfm = Node(
            ppt.fslaff2rigid(out_file="outputmatrix"), name="aff2rigid",
        )

        self.ACPC_warp = Node(
            fsl.preprocess.ApplyWarp(
                out_file="T1w_space-acpc.nii.gz",
                relwarp=True,
                output_type="NIFTI_GZ",
                interp="spline",
                ref_file=MNI_template,
            ),
            name="apply_warp",
        )

        self.t1_bet = Node(
            fsl.preprocess.BET(mask=True, robust=True, out_file="acpc_t1_brain.nii.gz"),
            name="fsl_bet",
        )

        self.epi_reg = Node(fsl.epi.EpiReg(out_base="dwi2acpc"), name="fsl_epireg",)

        self.acpc_xfm = Node(
            ppt.TransConvert(flirt=True, out_file="dwi2acpc_xfm.mat", force=True),
            name="transformconvert",
        )

        self.apply_xfm = Node(
            ppt.MRTransform(out_file="dwi_acpc.mif"), name="mrtransform",
        )

        self.regrid = Node(
            ppt.MRRegrid(
                out_file="dwi_space-acpc_res-1mm.mif",
                regrid=MNI_template,
            ),
            name="mrgrid",
        )

        self.gen_5tt = Node(
            ppt.Make5ttFSL(premasked=True, out_file="T1w_space-acpc_seg-5tt.mif"),
            name="mrtrix_5ttgen",
        )
        self.gmwmi = Node(ppt.gmwmi(out_file="gmwmi.nii.gz"), name="5tt2gmwmi")
        self.binarize_gmwmi = Node(
            ppt.MRThreshold(
                opt_abs=0.05, out_file="T1w_space-acpc_seg-gmwmi_mask.nii.gz"
            ),
            name="binarize_gmwmi",
        )
        self.convert2wm = Node(
            ppt.Convert(
                coord=[3, 2],
                axes=[0, 1, 2],
                out_file="T1w_space-acpc_seg-wm_mask.nii.gz",
            ),
            name="5tt2wm",
        )

# Cell
class PostProcNodes:
    """
    Inputs:
        BIDS_dir (str): Path to BIDS directory
        subj_template (dict): template directory for tck, dwi, T1, mask files
        sub_list (list): subjects IDs list generated from BIDS layout
        ses_list (list): sessions IDs listgenerated from BIDS layout
        skip_tuples (tuple): [('subject', 'session')] string pair to skip
    """

    def __init__(self, BIDS_dir, subj_template, sub_list, ses_list, skip_tuples):
        # filter & create sub-graphs for subjects and session combos:
        sub_iter, ses_iter = ppt.filter_workflow(BIDS_dir, sub_list, ses_list, skip_tuples)

        # Create BIDS output folder list for datasink
        BIDSFolders = [
            (
                "connectomes/ses-%ssub-%s" % (session, subject),
                "sub-%s/ses-%s/connectomes" % (subject, session),
            )
            for session in ses_list
            for subject in sub_list
        ]
        preproc_dir = os.path.join(
            BIDS_dir, "derivatives", "pipetography"
        )  # BIDS derivatives directory containing preprocessed outputs and streamline outputs

        # DWI input:
        self.subject_source = Node(
            IdentityInterface(fields=["subject_id", "session_id"]),
            iterables=[("subject_id", sub_iter), ("session_id", ses_iter)],
            synchronize=True,
            name="subj_source",
        )
        self.select_files = Node(
            SelectFiles(subj_template), base_directory=BIDS_dir, name="select_subjects"
        )
        self.linear_reg = Node(
            ants.Registration(
                output_transform_prefix="atlas_in_dwi_affine",
                dimension=3,
                collapse_output_transforms=True,
                transforms=["Affine"],
                transform_parameters=[(0.1,)],
                metric=["MI"],
                metric_weight=[1],
                radius_or_number_of_bins=[64],
                number_of_iterations=[[500, 200, 200, 100]],
                convergence_threshold=[1e-6],
                convergence_window_size=[10],
                smoothing_sigmas=[[4, 2, 1, 0]],
                sigma_units=["vox"],
                shrink_factors=[[8, 4, 2, 1]],
                use_histogram_matching=[True],
                output_warped_image="atlas_in_dwi_affine.nii.gz",
            ),
            name="linear_registration",
        )
        self.nonlinear_reg = Node(
            ants.Registration(
                output_transform_prefix="atlas_in_dwi_syn",
                dimension=3,
                collapse_output_transforms=True,
                transforms=["SyN"],
                transform_parameters=[(0.1,)],
                metric=["MI"],
                metric_weight=[1],
                radius_or_number_of_bins=[64],
                number_of_iterations=[[500, 200, 200, 100]],
                convergence_threshold=[1e-06],
                convergence_window_size=[10],
                smoothing_sigmas=[[4, 2, 1, 0]],
                sigma_units=["vox"],
                shrink_factors=[[8, 4, 2, 1]],
                use_histogram_matching=[True],
                output_warped_image="atlas_in_dwi_syn.nii.gz",
            ),
            name="nonlinear_registration",
        )
        self.round_atlas = Node(
            ppt.CheckNIZ(args="-round", out_file="nodes.mif"), name="round_parcellation"
        )
        self.response = Node(
            ResponseSD(
                algorithm="dhollander",
                wm_file="wm.txt",
                gm_file="gm.txt",
                csf_file="csf.txt",
            ),
            name="SDResponse",
        )
        self.fod = Node(
            ConstrainedSphericalDeconvolution(
                algorithm="msmt_csd",
                wm_txt="wm.txt",
                gm_txt="gm.txt",
                gm_odf="gm.mif",
                csf_txt="csf.txt",
                csf_odf="csf.mif",
            ),
            name="dwiFOD",
        )
        self.sift2 = Node(
            ppt.tckSIFT2(fd_scale_gm=True, out_file="sift2.txt"), name="sift2_filtering"
        )
        self.connectome = Node(
            ppt.MakeConnectome(
                out_file="connectome.csv", symmetric=True, zero_diag=True
            ),
            name="weight_connectome",
        )
        self.distance = Node(
            ppt.MakeConnectome(
                scale_length=True,
                stat_edge="mean",
                symmetric=True,
                zero_diag=True,
                out_file="distances.csv",
            ),
            name="weight_distance",
        )
        self.datasink = Node(DataSink(base_directory=preproc_dir), name="datasink")
        substitutions = [("_subject_id_", "sub-"), ("_session_id_", "ses-")]
        substitutions.extend(BIDSFolders)
        self.datasink.inputs.substitutions = substitutions
        self.datasink.inputs.regexp_substitutions = [
            (r"(_moving_image_.*\.\.)", ""),
            (r"(\.nii|\.gz)", ""),
        ]
        print("Data sink (output folder) is set to {}".format(preproc_dir))